/*============================================================================
  Summary:  Demonstrates Spinlock Contention in SQL Server
------------------------------------------------------------------------------
  Written by Klaus Aschenbrenner, www.SQLpassion.at

  For more scripts and sample code, check out 
    http://www.SQLpassion.at

  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- Start the .NET application which opens and hold 10.000 shared database locks in AdventureWorks2012
-- ...

-- Review the acquired locks
SELECT * FROM sys.dm_tran_locks
GO

-- Let's start now a workload, that acquires shared database locks in the same database
-- This will yield now to Spinlock Contention
-- ostress.exe -dAdventureWorks2012 -S"localhost\sql2014" -Q"{call sp_reset_connection()}" -n100 -r100000 -l120

-- The backoffs for the LOCK_HASH spinlock are now going up steadily...
SELECT * FROM sys.dm_os_spinlock_stats
WHERE name = 'LOCK_HASH'
GO

-- Retrieve the type value for the LOCK_HASH spinlock.
-- That value is used by the next XEvent session
SELECT * FROM sys.dm_xe_map_values
WHERE name = 'spinlock_types'
AND map_value = 'LOCK_HASH'
GO

-- Let's create an XEvent session that captures the backoff events, 
-- and bucketizes them on the callstack
CREATE EVENT SESSION SpinlockContention ON SERVER 
ADD EVENT sqlos.spinlock_backoff
(
    ACTION
	(
		package0.callstack
	)
	WHERE
	(
		[type] = 129 -- <<< Value from the previous query
	)
) 
ADD TARGET package0.histogram
(
	SET source = 'package0.callstack', source_type = 1
)
GO

-- Symbolize the call stack
DBCC TRACEON(-1, 2592, 3656)
GO

-- Start the XEvent session
ALTER EVENT SESSION SpinlockContention ON SERVER
STATE = START
GO

-- Clean up
DROP EVENT SESSION SpinlockContention ON SERVER 
GO